//
// Test codes for StringPool.
//
#include "szpool.h"

// For size_t and printf().
#include <cstdio>
// For strlen() and memcpy().
#include <cstring>

// For clock_t and clock().
#include <ctime>


namespace {

// Test StringPool::Pool.
template <typename Letter>
static void TestPool() {
  typedef StringPool::Pool<Letter> Pool;
  Pool *pool = new Pool(4096);

  // Make a string.
  char word[] = "Apple juice";
  Letter s[16];
  for (size_t i = 0; i < sizeof(word); ++i)
    s[i] = static_cast<Letter>(word[i]);

  printf("\n- StringPool::Pool (%d-bit)\n", sizeof(Letter) << 3);

  // Add a string many times.
  clock_t cl = clock();
  for (size_t i = 0; i < 1 << 20; ++i)
    pool->add(s);
  size_t num_strs = pool->num_strs();
  size_t total_size = pool->total_size();

  delete pool;

  cl = clock() - cl;
  printf("Time: %.3f sec\n", 1.0 * cl / CLOCKS_PER_SEC);
  printf("Number of strings: %u\n", num_strs);
  printf("Total size: %u\n", total_size);
}

// Test the case using a pair of new [] and delete[] for each string.
static void TestNewAndDelete() {
  char s[16] = "Apple juice";
  size_t size = strlen(s) + 1;

  printf("\n- new [] and delete [] for each char string\n");

  // Repeat new [] and delete [].
  clock_t cl = clock();
  for (size_t i = 0; i < 1 << 20; ++i) {
    char *buf = new char[size];
    memcpy(buf, s, size);
    delete [] buf;
  }

  cl = clock() - cl;
  printf("Time: %.3f sec\n", 1.0 * cl / CLOCKS_PER_SEC);
}

}  // namespace.


int main(void) {
  TestPool<char>();
  TestPool<short>();
  TestPool<int>();
  TestNewAndDelete();

  return 0;
}
